/* buffered.c  Demonstartion program for the WinSystems GXAD Linux Driver
*
*
*	$Header$
*
*	$Id$
*
*	$Logs$
*
*	Compile with :
*
*			gcc -static buffered.c kbhit.c gxadio.o -o buffered
*/

#include <stdio.h>
#include "gxad.h"


/* This program demonstrates a special feature of the device support library which
   allows for a programed sequence of conversions to be accomplished at speed,
   and the results are placed in a buffer for later processing.
   
   The input buffer is simply a list of channels to be converted in the desired order.
   The list is terminated with a hex FF value; The list is stroed in an 8-bit unsigned
   array and the results are stored in a 16-bit unsigned array.
   
   This program will load the input array such that 500 samples each will be made
   on channels 0 - 3 in succession.
   
*/

/* The to_do_channels array will hold the list of channels to be converted. Note
   that its length is one greater then the total number of desired conversions to 
   allow for the terminating 0xff character.
*/

__u8 to_do_channels[2001];

/* The raw results will be loaded into the values array. It's up to the use to know 
   how the resultant data is organized.
*/

__u16 values[2000];

int main(void)
{
int channel = 0;
__u16 result;
float min,max,current;
unsigned long count;
int x,c;

		/* First we'll set up the mode on all 8 channels to be single-ended, bipolar,
		   on a +/- 10V scale. This allows any legal input to be presented to any input pin.
		*/
		
		for(channel=0; channel <8; channel++)
		{
			gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_10V);
			
			/* If an error occured, gxad_error_code will be non-zero */
			
			if(gxad_error_code)
			{
				printf("%s - Aborting\n",gxad_error_string);
				exit(1);
			}
		}

		/* Fill up the to_do_array with 500 each for channels 0-3 */
					
		for(x=0; x<500; x++)
		{
			to_do_channels[x] = 0;
			to_do_channels[x+500] = 1;
			to_do_channels[x+1000] = 2;
			to_do_channels[x+1500] = 3;
			
		}
		
		/* Add the final terminating 0xff value to the to_do array */
		
		to_do_channels[2000] = 0xff;
		
		/* Enable raw keyboard mode */
		
		init_keyboard();
		
		/* Continue until a key is pressed */
		
		while(!kbhit())
		{
				/* Do the conversions. This function returns when all 2000
				   conversions are complete.
				*/
				
				gxad_buffered_channel_conversions(to_do_channels,values);
				
				/* Again. we check gcad_error_code to be sure it went as planned */
				
				if(gxad_error_code)
				{
					printf("%s - Aborting\n",gxad_error_string);
					close_keyboard();
					exit(1);
				}
				
				/* Now we'll extract our data from the values array. In order to make the
				   display more readable, we'll take a value from each channel, display them
				   and then move on to the next result.
				*/
				
				for(x=0; x<500; x++)
				{
					/* Get the raw data */
					
					result = values[x];
					
					/* Adjust for bipolar modes */
					
					result += gxad_adjust[0];
					
					/* Converto to voltage */
					
					current = result * gxad_bitval[0];
					
					/* One more adjustrment for bipolar modes */
					
					current += gxad_offset[0];
					
					/* Display the value */
					
					printf("CH0 %9.5f  ",current);
					
					/* Repeat for channels 1 -3 */
					
					result = values[x+500];
					result += gxad_adjust[1];
					current = result * gxad_bitval[1];
					current += gxad_offset[1];
					
					printf("CH1 %9.5f  ",current);

					result = values[x+1000];
					result += gxad_adjust[2];
					current = result * gxad_bitval[2];
					current += gxad_offset[2];
					
					printf("CH2 %9.5f  ",current);

					result = values[x+1500];
					result += gxad_adjust[3];
					current = result * gxad_bitval[3];
					current += gxad_offset[3];
					
					printf("CH3 %9.5f  ",current);
					printf("\r");
			}
		}
		
		/* Read the keypress */
		readch();
		
		/* Turn off raw keyboard mode */
		
		close_keyboard();
		printf("\n\n");
} 
						