/* getall.c Demonstration program for use with the WinSystems GXAD Linux driver
*
*	$Header$
*
*	$Id$
*
*	$Log$
*
*	Compile with :
*
*			gcc -static getall.c kbhit.c gxadio.o -o getall
*/

#include <stdio.h>
#include "gxad.h"

/* This program demonstrates usage of the gxad_convert_all_channels function call.
   This allows for all 8 channels to be quickly converted and the results passed
   back in an 8 element array. Note that this function does not return voltage values,
   it returns raw 16-bit data directly from the converter.
*/

/* Function prototypes */

void init_keyboard(void);
void close_keyboard(void);
int kbhit(void);
int readch(void);

/* This array will receive the result values for all 8 channels */

__u16 values[8];

int main(void)
{
int channel;
__u16 result;
float current;

		/* We set the mode on all 8 channels to single-ended bipolar +/1-V scale.
		   This allows for any device legal input voltages.
		*/
		
		for(channel=0; channel < 8; channel++)
		{			
			gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_10V);
			
			/* Check for an error by loooking at gxad_error_code */
			
			if(gxad_error_code)
			{
				printf("%s - Aborting\n",gxad_error_string);
				exit(1);
			}
		}
		
		/* Open the keyboard input for raw mode */
			
		init_keyboard();
		
		/* We'll go on until a key is pressed */
		
		while(!kbhit())
		{
			/* This is it! When this function returns the values from all 8 channels
			   will be present in the values array.
			*/
			
			gxad_convert_all_channels(values);
			
			/* Check for possible errors */
			
			if(gxad_error_code)
			{
				printf("%s - Aborting\n",gxad_error_string);
				close_keyboard();
				exit(1);
			}	
			
			/* Now we'll extract the data, convert it to volts, and display the results */
			
			for(channel =0; channel <8; channel++)
			{
				/* Get a result from the array */
				
				result = values[channel];

				/* Adjust for bipolar mode if necessary */
								
				result += gxad_adjust[channel];
			
				/* Convert it to a voltage */
					
				current = result * gxad_bitval[channel];
				
				/* A final adjustment for bipolar modes */
				
				current += gxad_offset[channel];

				/* Display the result */
							
				printf("CH%1d %5.2f ",channel,current);
			}
			printf("\r");
		}
		
		/* Clear the keypress */
		
		readch();

		/* Turn off raw input mode */
				
		close_keyboard();
		printf("\n\n");
}
