/* 
 * $Header$
 *
 * $Id$
 *
 * Filename : $RCSfile$
 * 
 * $Log$
 *
 *
 * WinSystems GXAD  Linux Device Driver
 */

 static char *RCSInfo = "$Id$";

/* Portions of original code Copyright (C) 1998-99 by Ori Pomerantz */


/* Standard in kernel modules */

#ifndef __KERNEL__
#  define __KERNEL__
#endif

#ifndef MODULE
#  define MODULE
#endif

#include <linux/config.h>
#include <linux/module.h>

#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/mm.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>

#include <asm/io.h>
#include <asm/uaccess.h>


MODULE_LICENSE("GPL");

//#define DEBUG 1

/* Our own ioctl numbers */

#include "gxad.h"


#define SUCCESS 0

unsigned base_port;

/* The name for our device, as it will appear in 
 * /proc/devices */

#define MAJOR_NUM 111
#define DEVICE_NAME "gxad"

/* The maximum length of the message for the device */

#define BUF_LEN 20

/* The message the device will give when asked */

static char Message[BUF_LEN];

/* A separate pointer for each device */

int Message_Ptr = 0;

/* Our insmod command line arguments */

MODULE_PARM(io,"1i");

static unsigned io = 0x1e8;;

int wait_ready(void);


/***********************************************************************
*
*
*			DEVICE OPEN
*
*
************************************************************************
*/


/* This function is called whenever a process attempts 
 * to open the device file */

static int device_open(struct inode *inode, struct file *file)
{

  unsigned int minor = MINOR(inode->i_rdev);

  if(minor >  0)
  {
	printk("<1>**** OPEN ATTEMPT on uninitialized port *****\n");
	return -1;
  }

#ifdef DEBUG
  printk ("device_open(%p)\n", file);
#endif

  /* Rewind the message pointer for this device on a new open.

     This could cause problems for applications that use file I/O
     for talking to the device as we do not limit the number of
     applications that can open the device. The solution is to use
     IOCTL functions to control the device rather than file I/O
     calls.
  */

  Message_Ptr  = 0;

  return SUCCESS;
}


/*************************************************************************
*
*
*			DEVICE CLOSE
*
*
**************************************************************************
*/


int device_release(struct inode *inode, struct file *file)
{

#ifdef DEBUG
  printk ("device_release(%p,%p)\n", inode, file);
#endif 


  return 0;

}


/****************************************************************************
*
*			DEVICE READ
*
*
*
*****************************************************************************
*/

ssize_t device_read(struct file *file,char *buffer,size_t length,loff_t *offset)
{
unsigned port;
int bytes_read;
int x;


   /* We only allow reading of the 10 valid ports in the device. Beyond
      that we return EOF.
   */

   if(Message_Ptr ==3)
	return 0;	/* EOF Condition */

   port = base_port;

   // Do a fresh read of the ports */

   for(x=0; x<3; x++)
       Message[x] = inb(port+x);


  /* Number of bytes actually written to the buffer */

   bytes_read = 0;

#ifdef DEBUG
  printk("device_read(%p,%p,%d)\n",file, buffer, length);
#endif

  /* Actually put the data into the buffer */

  while (length && (Message_Ptr < 3))
  {
    /* Because the buffer is in the user data segment, 
     * not the kernel data segment, assignment wouldn't 
     * work. Instead, we have to use put_user which 
     * copies data from the kernel data segment to the 
     * user data segment. */

    put_user((Message[Message_Ptr++]), buffer++);
    length --;
    bytes_read ++;
  }

#ifdef DEBUG
   printk ("Read %d bytes, %d left\n",bytes_read, length);
#endif

   /* Read functions are supposed to return the number 
    * of bytes actually inserted into the buffer */


  /* By updating file->f_pos we can allow seeking of our Messge_Ptr
     which will allow random access to the ports using file I/O
  */

  file->f_pos = (loff_t)Message_Ptr;

  return bytes_read;
}


/****************************************************************************
*
*
*				DEVICE WRITE
*
*****************************************************************************
*/

ssize_t device_write(struct file *file,const char *buffer,size_t length,loff_t *offset)
{
int i;
unsigned port;


    port = base_port;

#ifdef DEBUG
  printk ("device_write(%p,%s,%d)",file, buffer, length);
#endif

  /* Read in the bytes to write, one at a time */

  for(i=0; i<length && i<BUF_LEN; i++)
  {
    get_user(Message[i], buffer+i);

    /* We output each byte to it's appropriate port as it comes in */

    outb(Message[i],port + Message_Ptr++);
  }

  /* As with device_read if we update the file position. Seeking will
     work as expected.
  */

  file->f_pos = (loff_t) Message_Ptr;


  /* Again, return the number of input characters used */

    return i;
}


/****************************************************************************
*
*
*
*			DEVICE IOCTL
*
*
*****************************************************************************
*/


int device_ioctl(struct inode *inode,struct file *file,
     unsigned int ioctl_num, unsigned long ioctl_param)
{
int  ret_val;

#ifdef DEBUG
  printk("GXAD - IOCTL call IOCTL CODE %04X\n",ioctl_num);
#endif


  /* Switch according to the ioctl called */

  switch (ioctl_num)
  {

	/* Write a command to the A2D Command register */

    case IOCTL_COMMAND_WRITE:

		ret_val = ioctl_param & 0xff;
		outb(ret_val,io);

#ifdef DEBUG
		printk("Writing to Command  Port %04x with%02x\n", 
                  io,ret_val);
#endif

		return SUCCESS;
		break;

    case IOCTL_STATUS_READ:

		ret_val = inb(io);

#ifdef DEBUG
		printk("Reading Status port  %04x  result = %02x\n", 
                  io,ret_val);
#endif

		return ret_val;
		break;

    case IOCTL_DATA_READ:

		ret_val = inb(io+2);	/* Msb of data */
		ret_val = ret_val << 8;
		ret_val = ret_val | inb(io+1);     /* LSB of data */

#ifdef DEBUG
		printk("Reading Data port %04x  result = %02x\n",io+1,ret_val);
#endif

		return ret_val;
		break;

    case IOCTL_CONVERT_CURRENT:

#ifdef DEBUG
		printk("IOCTL command recognized - CONVERT_CURRENT\n");
#endif
	
		ret_val = ioctl_param & 0xff;

		outb(ret_val,io);
		if(wait_ready())
			return(-1);
		outb(ret_val,io);
		if(wait_ready())
			return(-1);

#ifdef DEBUG
		printk("Writing command port %04x with value %04x\n",io,ret_val);
#endif

		ret_val = inb(io+2);	/* Msb of data */
		ret_val = ret_val << 8;
		ret_val = ret_val | inb(io+1);  /* LSB of data */

#ifdef DEBUG
		printk("Reading Data port %04x  result = %04x\n",io+1,ret_val);
#endif

		return ret_val;
		break;
  }
  return 0;
}

/***************************** DEVICE_LSEEK ***************************/

loff_t device_lseek(struct file *filp, loff_t off, int whence)
{
unsigned offset;


    /* Convert the offset value to a more familiar size. After all
       our device is a fixed size of 3 bytes.
    */

    offset = (unsigned)off;

#ifdef DEBUG
    printk("<1>GXAD:lseek - offset %d mode %d\n",offset,whence);
#endif

    switch(whence)
    {
	case 0:   /* Seek set */
	    Message_Ptr = offset;
	    break;

	case 1:   /* Seek from Current */
	    Message_Ptr += offset;
	    break;
	
	default:	  /* Seek from End */
	    return -EINVAL;
	    break;
    }

	if(Message_Ptr < 0)
	{
	    Message_Ptr = 0;
	    filp->f_pos = (loff_t)Message_Ptr;
	    return -EINVAL;
	}

	if(Message_Ptr > 3)
	{
	    Message_Ptr = 3;
	    filp->f_pos = (loff_t)Message_Ptr;
	    return -EINVAL;
	}

	filp->f_pos = (loff_t)Message_Ptr;
	return (loff_t)Message_Ptr;
}

/* ******************* Module Declarations *************************** */


/* This structure will hold the functions to be called 
 * when a process does something to the our device 
*/

struct file_operations Fops = {

  owner:	THIS_MODULE,
  llseek:	device_lseek,
  read:		device_read, 
  write:	device_write,
  ioctl:	device_ioctl,
  open:		device_open,
  release:	device_release,
};

/****************************************************************************
*
*			INIT_MODULE
*
*
*****************************************************************************
*/

/* Initialize the module - Register the character device */

int init_module()
{
int ret_val;


  /* Sign-on */

   printk("<1>\nWinSystems GXAD Driver V1.00. Copyright 2005. All rights reserved\n");
   printk("<1>%s\n",RCSInfo);


  /* Check and Map our I/O region requests */

  if(io != 0)
  {
		if(request_region(io,0x4,"GXAD") == NULL)
		{
	   		printk("<1>Unable to use I/O Address %04X\n",io);
           io = 0;
    		return(-1);
		}
  }


  /* Register the character device */

  ret_val = register_chrdev(MAJOR_NUM, DEVICE_NAME, &Fops);

  /* Negative values signify an error */
 
	if (ret_val < 0)
	{
  		printk ("%s failed with %d\n",
            "Sorry, registering the character device ", ret_val);
    	return ret_val;
  	}

  	return 0;
}

/***************************************************************************
*
*			CLEANUP_MODULE
*
****************************************************************************
*/

/* Cleanup - unregister the appropriate file from /proc */

void cleanup_module()
{

  /* Unregister I/O Port usage */

     if(io)
     {
         release_region(io,0x4);
     }

  /* Unregister the device */

  unregister_chrdev(MAJOR_NUM, DEVICE_NAME);
 
}  


int wait_ready(void)
{
int retry;

	retry = 10000;
	while(retry--)
	{
		if((inb(io) & 1) == 0)
			return 0;
	}
	return 1;
}


