/* Filename : $RCSfile$
 *
 * The declarations here have to be in a header file, 
 * because they need to be known both to the kernel 
 * module (in chardev.c) and the process calling ioctl 
 * (ioctl.c)
 */

/* $Log$
*
*/

#include <linux/types.h>

#ifndef CHARDEV_H
#define CHARDEV_H

#include <linux/ioctl.h> 



#define IOCTL_SET_MSG _IOR(MAJOR_NUM, 0, char *)

/* _IOR means that we're creating an ioctl command 
 * number for passing information from a user process
 * to the kernel module. 
 *
 * The first arguments, MAJOR_NUM, is the major device 
 * number we're using.
 *
 * The second argument is the number of the command 
 * (there could be several with different meanings).
 *
 * The third argument is the type we want to get from 
 * the process to the kernel.
 */

/* Write a command byte to the GXAD  */

#define IOCTL_COMMAND_WRITE _IOWR(MAJOR_NUM, 1, int)

/* Read the A2D status port */

#define IOCTL_STATUS_READ _IOWR(MAJOR_NUM, 2, int)


/* Read out the conversion data  */

#define IOCTL_DATA_READ _IOWR(MAJOR_NUM,3, int)


/* Start a conversion, return the data  */

#define IOCTL_CONVERT_CURRENT  _IOWR(MAJOR_NUM, 4, int)

#define DEVICE_FILE_NAME "gxad"

#endif


#define GXAD_BASE_ADDRESS 0x1e8
#define GXAD_COMMAND_PORT GXAD_BASE_ADDRESS
#define GXAD_STATUS_PORT  GXAD_BASE_ADDRESS
#define GXAD_LSB_DATA	  GXAD_BASE_ADDRESS+1
#define GXAD_MSB_DATA	  GXAD_BASE_ADDRESS+2
#define GXAD_INT_CONTROL  GXAD_BASE_ADDRESS+2

#define GXAD_INT_BIT	0x80
#define GXAD_BUSY_BIT	0x01

#define GXAD_SINGLE_ENDED 0x80
#define GXAD_DIFFERENTIAL 0x00

#define GXAD_UNIPOLAR  0x08
#define GXAD_BIPOLAR   0x00

#define GXAD_TOP_5V 	0x00
#define GXAD_TOP_10V	0x04

#define GXAD_CH0_SELECT 0x00
#define GXAD_CH1_SELECT 0x40
#define GXAD_CH2_SELECT 0x10
#define GXAD_CH3_SELECT 0x50
#define GXAD_CH4_SELECT 0x20
#define GXAD_CH5_SELECT 0x60
#define GXAD_CH6_SELECT 0x30
#define GXAD_CH7_SELECT 0x70

#define GXAD_SUCCESS	0
#define GXAD_OPEN_ERROR 1
#define GXAD_TIMEOUT_ERROR 2
#define GXAD_BAD_CHANNEL_NUMBER 3
#define GXAD_BAD_MODE_NUMBER 4
#define GXAD_BAD_RANGE 5
#define GXAD_COMMAND_WRITE_FAILURE 6
#define GXAD_READ_DATA_FAILURE 7


int gxad_open_device(void);
int gxad_write_comnand(__u8 command);
__u8 gxad_read_status(void);
__u16 gxad_read_conversion_data(void);
int gxad_start_conversion(int channel);
int gxad_set_channel_mode(int channel,int input_mode,int duplex,int range);
float gxad_auto_get_channel_voltage(int channel);
int gxad_wait_ready(void);
float gxad_get_channel_voltage(int channel);
__u16 gxad_convert_current(int channel);
int gxad_convert_single_repeated(int channel,__u16 count,__u16 *buffer);
int gxad_buffered_channel_conversions(__u8 *input_channel_buffer,__u16 *buffer);
int gxad_convert_all_channels(__u16 *buffer);

#ifdef LIB_DEFINED
__u8 gxad_channel_select[9] = {GXAD_CH0_SELECT, GXAD_CH1_SELECT,
		GXAD_CH2_SELECT, GXAD_CH3_SELECT, GXAD_CH4_SELECT,
		GXAD_CH5_SELECT, GXAD_CH6_SELECT, GXAD_CH7_SELECT };

__u16 gxad_channel_mode[8] = { GXAD_CH0_SELECT, GXAD_CH1_SELECT,
		GXAD_CH2_SELECT, GXAD_CH3_SELECT, GXAD_CH4_SELECT,
		GXAD_CH5_SELECT, GXAD_CH6_SELECT, GXAD_CH7_SELECT };

float gxad_bitval[8] = { .0001, .0001, .0001, .0001, .0001, .0001,
 			.0001, .0001 };
  
__u16 gxad_adjust[8] = {0,0,0,0,0,0,0,0};
float gxad_offset[8] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };

int gxad_buff_mode = 0;
int gxad_last_channel = 0xff;
int gxad_current_channel = 0xff;

int gxad_repeat_mode = 0;
int gxad_repeat_count = 0;
int gxad_repeat_channel = 0;
int gxad_repeat = 0;

__u16 gxad_channel_buff[10] = { 1,2,3,4,5,6,7,0,0xff,0xff};
__u8 *gxad_input_buffer;
__u16 *gxad_user_buffer;
int gxad_ch_index = 0;
int gxad_out_index = 0;

int gxad_error_code;
char gxad_error_string[128];

#else

extern __u8 gxad_channel_select[9];
extern __u8 gxad_channel_mode[8];
extern float gxad_bitval[8];
extern __u16 gxad_adjust[8];
extern float gxad_offset[8];
extern int gxad_buff_mode;
extern int gxad_last_channel;
extern int gxad_current_channel;
extern int gxad_repeat_mode;
extern int gxad_repeat_count;
extern int gxad_repeat_channel;
extern int gxad_repeat;
extern __u16 gxad_channel_buff[10];
extern __u8 *gxad_input_buffer;
extern __u16 *gxad_user_buffer;
extern int gxad_ch_index;
extern int gxad_out_index;

extern int gxad_error_code;
extern char gxad_error_string[];

#endif



