/* uio48.h WinSystems header file for UIO68 Linux driver */
/*
*  $Header$
*
*  $Id$
*
*  $Log$
*
*/

#define LIB_DEFINED 1

#include <stdio.h>
#include <fcntl.h>      /* open */ 
#include <unistd.h>     /* exit */
#include <sys/ioctl.h>  /* ioctl */

#include "gxad.h"    

//#define DEBUG 1

static int init_complete = 0;

/**************************************************************************
*
*		USER LIBRARY FUNCTIONS
*
***************************************************************************
*/

int handle = 0;
char *device_id ="gxad";


/**************************************************************************************
*
*		GXAD_AUTO_GET_CHANNEL_VOLTAGE
*
***************************************************************************************
*/
float gxad_auto_get_channel_voltage(int channel)
{
float result;
__u16 value;
int val;

#ifdef DEBUG
	printf("GXAD : Entering function gxad_auto_get_channel_voltage\n");
#endif

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(0.0);

	/* Start out on an +/10V scale */

	val = gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_10V);
	if(gxad_error_code)
		return(0.0);
		
	value = gxad_convert_current(channel);
	if(gxad_error_code)
		return(0.0);
		
#ifdef DEBUG
	printf("GXAD_AUTO_GET_CHANNEL_VOLTAGE : First conversion raw data = %04x\n",value);
#endif

	/* Convert raw value to voltage */

	value =  (value + gxad_adjust[channel]) & 0xffff;
	result = value * gxad_bitval[channel];
	result += gxad_offset[channel];

#ifdef DEBUG
	printf("GXAD_AUTO_GET_CHANNEL_VOLTAGE : gxad_adjust = %04x gxad_bitval = %9.5f gxad_offset = %9.5f\n",
			gxad_adjust[channel],gxad_bitval[channel],gxad_offset[channel]);
#endif

	/* If the voltage is less than -5.0 volts. The range just used
	   is the correct one, so return the reading
	*/
#ifdef DEBUG
	printf("GXAD_AUTO_GET_CHANNEL_VOLTAGE : First voltage reading = %9.5f\n",result);
#endif

	if(result < -5.00)
		return result;

	/* If the result is between -4.99 and 0.0 we can switch to the
	   +/- 5 Volt scale for a more precise reading.
	*/

	if(result < 0.0)
		gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_5V);

	if(gxad_error_code)
		return(0.0);
			
	/* If the value is above 5 volts the 0 - 10 volt scale will give our most
	   precise result.
	*/

	if(result >= 5.00)
		gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_UNIPOLAR,GXAD_TOP_10V);

	if(gxad_error_code)
		return(0.0);
			
	/* Lastly, If we're greater than zero and less than 5.0 volts, the 0 - 5 volt
	   scla is the best choice.
	*/

	if((result > 0.0) && (result < 5.00))
		gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_UNIPOLAR,GXAD_TOP_5V);

	if(gxad_error_code)
		return(0.0);	

	/* Now that we've got the range chosen. Read the current data using the new
           scale.
	*/

	value = gxad_convert_current(channel);

	if(gxad_error_code)
		return(0.0);
		
#ifdef DEBUG
	printf("GXAD_AUTO_GET_CHANNEL_VOLTAGE : Second conversion raw data = %04x\n",value);
#endif
	
	/* Convert raw value to voltage */

#ifdef DEBUG
	printf("GXAD_AUTO_GET_CHANNEL_VOLTAGE : gxad_adjust = %04x gxad_bitval = %9.5f gxad_offset = %9.5f\n",
			gxad_adjust[channel],gxad_bitval[channel],gxad_offset[channel]);
#endif
	
	value =  (value + gxad_adjust[channel]) & 0xffff;
	result = value * gxad_bitval[channel];
	result += gxad_offset[channel];

#ifdef DEBUG
	printf("GXAD_AUTO_GET_CHANNEL_VOLTAGE : result = %9.5f\n",result);
#endif
	
	return((float)result);
}

/*******************************************************************************************
*
*			GXAD_GET_CHANNEL_VOLTAGE
*
********************************************************************************************
*/

float gxad_get_channel_voltage(int channel)
{
__u16 value;
float result;

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(0.0);


	value = gxad_convert_current(channel);

	if(gxad_error_code)
		return(0.0);
		
	/* Convert raw value to voltage */

	value += gxad_adjust[channel];
	result = value * gxad_bitval[channel];
	result += gxad_offset[channel];

	return result;
}
	
/***********************************************************************************************
*
*			GXAD_BUFFERED_CHANNEL_CONVERSIONS
*
************************************************************************************************
*/
int gxad_buffered_channel_conversions(__u8 *input_channel_buffer, __u16 *buffer)
{

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(-1);
	
	/* Transfer the address of the user supplied buffer to our own glable pointers */

	gxad_user_buffer = buffer;
	gxad_input_buffer = input_channel_buffer;

	/* Reset the array index pointers */

	gxad_ch_index = 0;
	gxad_out_index = 0;


	gxad_start_conversion(gxad_input_buffer[gxad_ch_index++]);
	if(gxad_error_code)
		return(-1);
		
	gxad_wait_ready();
	if(gxad_error_code)
		return(-1);
		
	/* While there are channel numbers in the buffer ( != 0xff)
	   convert the requested channel and place the results in
	   the the user's output buffer.
	*/

	while(gxad_input_buffer[gxad_ch_index] != 0xff)
	{
		gxad_start_conversion(gxad_input_buffer[gxad_ch_index++]);
		if(gxad_error_code)
			return(-1);
		gxad_wait_ready();
		if(gxad_error_code)
			return(-1);
		gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
	}

	/* One last conversion allows us to retrieve our real last data */

	gxad_start_conversion(0);	/* Channel number irrelevant */
	if(gxad_error_code)
		return(-1);
	gxad_wait_ready();
	if(gxad_error_code)
		return(-1);
		
	gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();

	return 0;
}


/******************************************************************************************************
*
*			GXAD_CONVERT_SINGLE_REPEATED
*
******************************************************************************************************
*/

int gxad_convert_single_repeated(int channel, __u16 count, __u16 *buffer)
{
int x;

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(-1);

	gxad_user_buffer = buffer;
	gxad_out_index = 0;
	gxad_repeat_channel = channel;
	gxad_repeat_count = count;


	gxad_start_conversion(gxad_repeat_channel);
	if(gxad_error_code)
		return(-1);
	gxad_wait_ready();
	if(gxad_error_code)
		return(-1);
		
	/* Perform the requested number of conversions. Place the results into
	   the user's output buffer.
	*/

#ifdef DEBUG
	printf("GXAD_CONVERT_SINGLE_REPEATED : Performing %d repeated conversions\n",count);
#endif

	for(x=1; x<=gxad_repeat_count; x++)
	{
		gxad_start_conversion(gxad_repeat_channel);
		if(gxad_error_code)
			return(-1);
		gxad_wait_ready();
		if(gxad_error_code)
			return(-1);
			
		gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
		if(gxad_error_code)
			return(-1);
	}

	/* One last dummy conversion to retrieve our last conversion */

		gxad_start_conversion(gxad_repeat_channel);
		if(gxad_error_code)
			return(-1);
		gxad_wait_ready();
		if(gxad_error_code)
			return(-1);
		gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
		if(gxad_error_code)
			return(-1);
			
#ifdef DEBUG
	printf("GXAD_CONVERT_SINGLE_REPEATED : %d conversions Completed\n",count);
#endif
		
	return 0;
}
		
/***********************************************************************************************
*
*			GXAD_CONVERT_ALL_CHANNELS
*
***********************************************************************************************
*/

int gxad_convert_all_channels(__u16 *buffer)
{
int x;

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(-1);

	/* Initializa the global variables including transferring the address of
	   the user's output buffer to an internal buffer pointer.
	*/

	gxad_user_buffer = buffer;
	gxad_ch_index = 0;
	gxad_out_index = 0;

	gxad_start_conversion(0);
	if(gxad_error_code)
		return(-1);
	gxad_wait_ready();
	if(gxad_error_code)
		return(-1);
		
	/* Finish reading the rest of the channels */

	for(x=1; x<8; x++)
	{
		gxad_start_conversion(x);
		if(gxad_error_code)
			return(-1);
		gxad_wait_ready();
		if(gxad_error_code)
			return(-1);

		/* Store the results in the user's output buffer */

		gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
		if(gxad_error_code)
			return(-1);
	}

	/* A final dummy conversion is required to get the last of our data */

	gxad_start_conversion(0);
	if(gxad_error_code)
		return(-1);
	gxad_wait_ready();
	if(gxad_error_code)
		return(-1);
		
	gxad_user_buffer[gxad_out_index++] = gxad_read_conversion_data();
	if(gxad_error_code)
		return(-1);
		
	return 0;
}

/**************************************************************************************************
*
*			GXAD_SET_CHANNEL_MODE
*
**************************************************************************************************
*/

int gxad_set_channel_mode(int channel, int input_mode, int duplex, int range)
{
__u8 command_byte;

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(-1);

	/* Check for legal channel numbers */

	if((channel < 0) || (channel > 7))
	{	
		gxad_error_code = GXAD_BAD_CHANNEL_NUMBER;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Bad channel number %d",channel);	
		return(-1);
	}
	
	/* Check for illegal mode combinations */

	if((input_mode != GXAD_SINGLE_ENDED) && (input_mode != GXAD_DIFFERENTIAL))
	{	
		gxad_error_code = GXAD_BAD_MODE_NUMBER;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Illegal Mode specification");
		return(-1);
	}
		
	if((duplex != GXAD_UNIPOLAR) && (duplex != GXAD_BIPOLAR))
	{
		gxad_error_code = GXAD_BAD_MODE_NUMBER;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Illegal Mode specification");
		return(-1);
	}
	
	if((range != GXAD_TOP_5V) && (range != GXAD_TOP_10V))
	{
		gxad_error_code = GXAD_BAD_RANGE;
		sprintf(gxad_error_string,"GXAD : Set Channel Mode - Illegal Range Specification");
		return(-1);
	}

	/* Build a command byte out of the arguments */

	command_byte = gxad_channel_select[channel];

	command_byte = command_byte | input_mode | duplex | range;

	gxad_channel_mode[channel] = command_byte;


	/* Now calculate bit values, offsets and adjustment values */

	if((range == GXAD_TOP_5V) && (duplex == GXAD_UNIPOLAR))
	{
		gxad_bitval[channel] = 5.000 / 65536.0;
		gxad_adjust[channel] = 0;
		gxad_offset[channel] = 0.0;
	}

	if((range == GXAD_TOP_5V) && (duplex == GXAD_BIPOLAR))
	{
		gxad_bitval[channel] = 10.000 / 65536.0;
		gxad_adjust[channel] = 0x8000;
		gxad_offset[channel] = -5.00;
	}

	if((range == GXAD_TOP_10V) && (duplex == GXAD_UNIPOLAR))
	{
		gxad_bitval[channel] = 10.000 / 65536.0;
		gxad_adjust[channel] = 0;
		gxad_offset[channel] = 0.0;
	}

	if((range == GXAD_TOP_10V) && (duplex == GXAD_BIPOLAR))
	{
		gxad_bitval[channel] = 20.000 / 65536.0;
		gxad_adjust[channel] = 0x8000;
		gxad_offset[channel] = -10.0;
	}

	return 0;
}

/***********************************************************************************************
*
*		GXAD_START_CONVERSION
*
***********************************************************************************************
*/

int gxad_start_conversion(int channel)
{
int c;

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(-1);


    c=ioctl(handle,IOCTL_COMMAND_WRITE,gxad_channel_mode[channel]);

	if(c)
	{
		gxad_error_code = GXAD_COMMAND_WRITE_FAILURE;
		sprintf(gxad_error_string,"GXAD : IOCTL_COMMAND_WRITE failure");
		return(-1);
	}
	
    return 0;
}


/*****************************************************************************************************
*
*		GXAD_READ_CONVERSION_DATA
*
*****************************************************************************************************
*/
__u16  gxad_read_conversion_data(void)
{
__u16 c;

	gxad_error_code = GXAD_SUCCESS;
	
	if(!init_complete)
		if(gxad_open_device())
			return(-1);

	
    c=ioctl(handle,IOCTL_DATA_READ,0);

	return  c;

}

__u8 gxad_read_status(void)
{
__u8 c;

	if(!init_complete)
		gxad_open_device();
		
    c=ioctl(handle,IOCTL_STATUS_READ,0);

    return  c;
}
		
/******************************************************************************************************
*
*		GXAD_CONVERT_CURRENT
*
*******************************************************************************************************
*/
__u16 gxad_convert_current(int channel)
{
__u16 c;
   
   if(!init_complete)
	gxad_open_device();		

    c=ioctl(handle,IOCTL_CONVERT_CURRENT,gxad_channel_mode[channel]);

    return c;
}


/***************************************************************************************************
*
*		GXAD_WAIT_READY
*
****************************************************************************************************
*/
int gxad_wait_ready(void)
{
unsigned retry;
__u8 val;

//	return 0;
	
	retry = 1000;
	while(retry--)
	{
		val = gxad_read_status();
		if((val & GXAD_BUSY_BIT) == 0)
			return 0;
	}
	gxad_error_code = GXAD_TIMEOUT_ERROR;
	sprintf(gxad_error_string,"GXAD : Device timeout error");
	
	return 1;
}

/****************************************************************************************************
*
*		GXAD_OPEN_DEVICE
*
*****************************************************************************************************
*/
int gxad_open_device(void)
{

#ifdef DEBUG
	printf("GXAD : Entering function gxad_open_device\n");
#endif

    if(init_complete)
		return 0;

    if(handle == -1)	/* If it's already been tried */
	{
		gxad_error_code = GXAD_OPEN_ERROR;
		sprintf(gxad_error_string,"GXAD : Unable to open GXAD device");
		return -1;
	}
    /* Try opening the device file, in case it hasn't been opened yet */

#ifdef DEBUG
	printf("GXAD : Preparing to call open on device node :%s\n",device_id);
#endif	
    handle = open(device_id,0);
#ifdef DEBUG
	printf("GXAD : Return from open. Handle = %d\n",handle);
#endif
    if(handle > 0)	/* If it's now a valid handle */
    {
		++init_complete;
		return 0;
    }

	gxad_error_code = GXAD_OPEN_ERROR;
	sprintf(gxad_error_string,"GXAD : Unable to open GXAD device\n");

    handle = -1;
    return -1;
}

