/* repeat.c A demo program for the Winsystems GXAD Linux driver
*
*
*	$Header$
*
*	$Id$
*
*	$Log$
*
*/

#include <stdio.h>
#include "gxad.h"

	/* This program demonstrates the gxad_convert_single_repeated function call in
	   the support library. This function allows for a specified number of conversions
	   to be completed and the results placed in a buffer before returning. Note :
	   This function returns raw 16-bit values and it does not return any voltage
	   values.
	*/
	
	
void init_keyboard(void);
void close_keyboard(void);
int kbhit(void);
int readch(void);

	/* This array will store the results of the 2000 conversions */
	
__u16 values[2000];

int main(int argc, char *argv[])
{
int channel = 0;
__u16 result;
float min,max,current;
unsigned long count;
int c,x;

	/* We'll keep track of the minimum and maximum voltage values we see on
	each channel as well as the count of conversions completed.
	*/
	
	max = -10.0;
	min = 10.0;
	count = 0;
	
	/* The default channel is channel 0 but an alternate channel can be specified with
		a command line argument
	*/
	
	if(argc > 1)
		channel = atoi(argv[1]);
		
	/* Check for a valid channel number - Abort if bad */
	
	if(channel < 0 || channel > 7)
	{
		printf("Channel numbers must be between 0 and 7 - Aborting\n");
		exit(0);
	}
	
	/* This call sets the mode for the specified channel. We are going to set
	   up for a single-ended +/- 10V range. THat way any legal input can be
	   accomodated.
	*/
	
	gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_10V);
	
	/* Check for errors to be sure the mode was set correctly */
	
	if(gxad_error_code)
	{
		printf("%s - Aborting\n",gxad_error_string);
		exit(1);
	}
	
	/* Open the keyboard for raw data input */
		
	init_keyboard();
	
	/* We'll run until a non-recognized keypress occurs */
	
	while(1)
	{
		if(kbhit())
		{
			c = readch();
	
			/* The 'C' key clears the min/max values and the count */
								
			if(c == 'c' || c == 'C')
			{
				count = 0;
				min = 10.0;
				max = -10.0;
			}
			
			/* The 'N' key moves to the next channel */
			
			else if(c == 'n' || c == 'N')
			{
				printf("\n");
				channel = (channel + 1) & 0x07;
				gxad_set_channel_mode(channel,GXAD_SINGLE_ENDED,GXAD_BIPOLAR,GXAD_TOP_10V);
				if(gxad_error_code)
				{
					printf("%s - Aborting\n",gxad_error_string);
					close_keyboard();
					exit(1);
				}
				count = 0;
				min = 10.0;
				max = -10.0;
			}
			
			/* Any other key, exits the program */
			
			else
			{
				close_keyboard();				
				printf("\n\n");
				exit(0);
			}
				
		}	
		
		/* Here's the call to the gxad_convert_single_repeated function */
		
		gxad_convert_single_repeated(channel,2000,values);
		
		/* Check for an error condition */
		
		if(gxad_error_code)
		{
			printf("%s - Aborting\n",gxad_error_string);
			exit(1);
		}
		
		/* Bump up the count */
			
		count += 2000;
		
		/* Extract the data, convert it to volts */
		
		for(x=0; x<2000; x++)
		{
			result = values[x];
		
			/* Adjust for bipolar modes */
				
			result = result + gxad_adjust[channel];
			
			/* Convert to a voltage */
			
			current = result * gxad_bitval[channel];
			
			/* Another adjustment for bipolar modes */
			
			current += gxad_offset[channel];
			
			/* Set the min/max values as appropriate */
			
			if(current < min)
				min = current;
			if(current > max)
				max = current;
			
			/* Display the results */
				
			printf("CH %1d %09ld %9.5f Min =%9.5f Max =%9.5f\r",channel,count,current,min,max);
		}
	}
}
